use super::datatypes::Request;

impl Request<'_> {
    /// Sets the `mime_type` of this [`Request`] from the headers.
    ///
    /// The mime_type can remain none if there isn't a `Content-Type: ` header
    ///
    /// # Example
    /// ```
    /// use http::{
    ///     handling::{methods::Method, request::Request},
    ///     utils::mime::mime_enum::Mime,
    /// };
    ///
    /// let mut request = Request {
    ///     uri: "thing",
    ///     headers: vec![
    ///         "GET / 23".to_string(),
    ///         "SDF:LKJSD:F".to_string(),
    ///         "Content-Type: text/plain".to_string(),
    ///         "SDF".to_string(),
    ///     ],
    ///     method: Method::Get,
    ///     cookies: None,
    ///     mime_type: None,
    /// };
    /// let mut wrong = Request {
    ///     uri: "thing",
    ///     headers: vec![
    ///         "GET / 23".to_string(),
    ///         "SDF:LKJSD:F".to_string(),
    ///         "SDF".to_string(),
    ///     ],
    ///     method: Method::Get,
    ///     cookies: None,
    ///     mime_type: None,
    /// };
    /// request.mime_from_headers();
    /// wrong.mime_from_headers();
    /// assert_eq!(None, wrong.mime_type);
    /// assert_eq!(Mime::TextPlain, request.mime_type.unwrap());
    /// ```
    /// # Panics
    /// No Panics
    pub fn mime_from_headers(&mut self) {
        let Some(content_type_header) = self.headers
            .iter()
            .find(|header| header.starts_with("Content-Type: ")) else {
            return;
        };
        let content_type_string = content_type_header
            .strip_prefix("Content-Type: ")
            .unwrap()
            .to_string();

        self.mime_type = match content_type_string.split_once(';') {
            Some(sub) => sub.0.trim().parse().ok(),
            None => content_type_string.trim().parse().ok(),
        };
    }
}

#[cfg(test)]
mod test {
    use crate::{
        handling::{methods::Method, request::Request},
        utils::mime::mime_enum::Mime,
    };

    #[test]
    pub fn test_mime_parse_from_header_vec() {
        let mut request = Request {
            uri: "thing",
            headers: vec![
                "GET / 23".to_string(),
                "SDF:LKJSD:F".to_string(),
                "Content-Type: text/plain".to_string(),
                "SDF".to_string(),
            ],
            method: Method::Get,
            cookies: None,
            mime_type: None,
        };

        let mut wrong = Request {
            uri: "thing",
            headers: vec![
                "GET / 23".to_string(),
                "SDF:LKJSD:F".to_string(),
                "SDF".to_string(),
            ],
            method: Method::Get,
            cookies: None,
            mime_type: None,
        };
        request.mime_from_headers();
        wrong.mime_from_headers();
        assert_eq!(None, wrong.mime_type);
        assert_eq!(Mime::TextPlain, request.mime_type.unwrap());
    }
}
